package at.jku.students.map;

import at.jku.ssw.Iterator;

import at.jku.ssw.list.impl.LinkedList;
import at.jku.ssw.map.HashMapUtil;
import at.jku.ssw.map.Map;
import at.jku.ssw.map.MapIterator;
import at.jku.ssw.map.Pair;
import at.jku.ssw.map.PairList;

/**
 * An implementation of the {@linkplain Map} class based on the
 * {@linkplain Object#hashCode()} of the key objects that uses linear probing as
 * its collision strategy.
 */
public final class HashMapLinearProbing extends Map {

	protected final int tabSize;
	protected Pair[] table;
	private int size;

	public HashMapLinearProbing(int size) {
		this.tabSize = size;
		table = new Pair[tabSize];
	}

	private int hash(Object val) {
		return HashMapUtil.mod(val.hashCode(), tabSize);
	}

	private static boolean isDeleted(Object o) {
		return o == Pair.DELETED_MARK;
	}

	/**
	 * Returns true if this map contains an association for the specified key.
	 *
	 * @throws NullPointerException
	 *             if the provided key is null.
	 */
	@Override
	public boolean containsKey(Object key) {
		// TODO
		return false;
	}

	/**
	 * Retrieves the value associated with the specified key.
	 *
	 * Returns null if the map contains no mapping for this key. A return value
	 * of null does not necessarily indicate that the map contains no mapping
	 * for the key; it's also possible that the map explicitly maps the key to
	 * null. The containsKey operation can be used to distinguish these two
	 * cases.
	 *
	 * @throws NullPointerException
	 *             if the provided key is null.
	 */
	@Override
	public Object get(Object key) {
		// TODO
		return null;
	}

	/**
	 * Creates an association of the specified key to the specified value in
	 * this map. If an association for the specified key is already present, it
	 * is replaced.
	 *
	 * @throws NullPointerException
	 *             if the provided key is null.
	 * @throws IllegalStateException
	 *             if the map is full.
	 */
	@Override
	public void put(Object key, Object value) {
		// TODO
	}

	/**
	 * If this map contains an association for the specified key, removes it and
	 * returns the formerly associated value.
	 *
	 * @throws NullPointerException
	 *             if the provided key is null.
	 */
	@Override
	public Object remove(Object key) {
		// TODO
		return null;
	}

	/**
	 * Returns the number of elements stored in this map.
	 * 
	 * @return the number of elements in this hashmap.
	 */
	@Override
	public int size() {
		// TODO
		return -1;
	}

	@Override
	public Iterator iterator() {
		LinkedList l = new LinkedList();
		for (int i = 0; i < tabSize; i++) {
			Pair e = table[i];
			if (e != null && !isDeleted(e)) {
				l.insertLast(new Pair(e.key, e.value));
			}
		}
		return new MapIterator(l);
	}

}
